# -*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement


from datetime import datetime
import inspect
import time
import functools
from collections import defaultdict

from turbo.util import import_object

from bson.objectid import ObjectId
from pymongo import ASCENDING, DESCENDING, collection

from turbo.log import model_log
from turbo.util import escape as _es


_record = lambda x: defaultdict(lambda: None, x)


def convert_to_record(func):
    """wrap mongodb record to a dict record

    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        if result is not None:
            if isinstance(result, dict):
                return _record(result)
            
            return (_record(i) for i in result)

        return result

    return wrapper


class MixinModel(object):

    @staticmethod
    def utctimestamp(seconds=None):
        if seconds:
            return long(time.mktime(time.gmtime(seconds)))
        else:
            return long(time.mktime(time.gmtime()))

    @staticmethod
    def timestamp():
        return long(time.time())

    @staticmethod
    def datetime(dt=None):
        if dt:
            return datetime.strptime(dt, '%Y-%m-%d %H:%M')
        else:
            return datetime.now()

    @staticmethod
    def utcdatetime(dt=None):
        if dt:
            return datetime.strptime(dt, '%Y-%m-%d %H:%M')
        else:
            return datetime.utcnow()

    @classmethod
    def to_one_str(cls, value, *args, **kwargs):
        """string化的单个record对象
        """
        if kwargs.get('wrapper'):
            return cls._wrapper_to_one_str(value)

        return _es.to_dict_str(value)

    @classmethod
    def to_str(cls, values, callback=None):
        """string化的多个record组成的list对象
        """
        if callback and callable(callback):
            if isinstance(values, dict):
                return callback(_es.to_str(values))

            return [callback(_es.to_str(i)) for i in values]
            
        return _es.to_str(values)

    @staticmethod
    @convert_to_record
    def _wrapper_to_one_str(value):
        return _es.to_dict_str(value)

    @staticmethod
    def default_encode(v):
        return _es.default_encode(v)

    @staticmethod
    def json_encode(v):
        return _es.json_encode(v)

    @staticmethod
    def json_decode(v):
        return _es.json_decode(v)

    @staticmethod
    def to_objectid(objid):
        return _es.to_objectid(objid)

    @staticmethod
    def create_objectid():
        """return ObjectId
        """
        return ObjectId()

    _instance = {}

    @classmethod
    def instance(cls, name):
        """
        instance application model
        """
        if not cls._instance.get(name):
            model_name = name.split('.')
            ins_name = '.'.join(['models', model_name[0], 'model', model_name[1]])
            cls._instance[name] = cls.import_model(ins_name)()

        return cls._instance[name]

    @classmethod
    def import_model(cls, ins_name):
        try:
            package_space = getattr(cls,  'package_space')
        except AttributeError:
            raise NotImplementedError
        else:
            return import_object(ins_name, package_space)

    @staticmethod
    def default_record():
        """
        generate one default record which return '' when key is empty 
        """
        return defaultdict(lambda: '')


class BaseBaseModel(MixinModel):
    """mongodb 基础api 访问

    name = None                            mongodb collection name
    field = None                           collection key
    column = None                          need to query field
    """

    _operators = {
        '$set': '',
        '$unset': '',
        '$rename': '',
        '$currentDate': '',
        '$inc': '',
        '$max': '',
        '$min': '',
        '$mul': '',
        '$setOnInsert': '',

        '$addToSet': '',
        '$pop': '',
        '$pushAll': '',
        '$push': '',
        '$pull': '',
    }

    def __init__(self, db_name='test', _MONGO_DB_MAPPING=None):
        if _MONGO_DB_MAPPING is None:
            raise Exception("db mapping is invalid")

        # databases
        self.__db = _MONGO_DB_MAPPING['db']
        # databases file
        self.__db_file = _MONGO_DB_MAPPING['db_file']

        #databse name
        if db_name not in self.__db or self.__db.get(db_name, None) is None:
            raise Exception("%s is invalid databse" % db_name)

        # collection name
        if not self.name:
            raise Exception("%s is invalid collection name" % self.name)

        # collection field
        if not self.field or not isinstance(self.field, dict):
            raise Exception("%s is invalid collection field" % self.field)

        # collect as private variable
        self.__collect = getattr(self.__db.get(db_name, object), self.name, None)
        if self.__collect is None:
            raise Exception("%s is invalid collection" % self.name)

        # gridfs as private variable
        self.__gridfs = self.__db_file.get(db_name, None)
        if self.__gridfs is None:
            model_log.warning("%s is invalid gridfs" % self.__gridfs)

    def __setitem__(self, k, v):
        setattr(self, k, v)

    def __getitem__(self, k):
        return getattr(self, k)
    
    def __getattr__(self, k):
        attr = getattr(self.__collect, k)
        if isinstance(attr, collection.Collection):
            raise AttributeError("model object '%s' has not attribute '%s'"%(self.name, k))

        return attr

    def sub_collection(self, name):
        return self.__collect[name] 

    def __str__(self):
        if isinstance(self.field, dict):
            return unicode(self.field)

        return None

    # pymongo collection method
    def insert(self, doc_or_docs, **kwargs):
        """collection insert method
        args:
            w(optional):(integer or string) If this is a replica set,
            write operations will block until they have been replicated to the specified number or tagged set of servers.
            w=<int> always includes the replica set primary (e.g. w=3 means write to the primary and wait until replicated to two secondaries).
            Passing w=0 disables write acknowledgement and all other write concern options.

            wtimeout(optional): (integer) Used in conjunction with w.
            Specify a value in milliseconds to control how long to wait for write propagation to complete.
            If replication does not complete in the given timeframe, a timeout exception is raised.

        """
        check = kwargs.pop('check', False)
        if check is True:
            return self.create(doc_or_docs, **kwargs)

        return self.__collect.insert(doc_or_docs, **kwargs)

    def save(self, to_save, **kwargs):
        """collection save method

        """
        return self.__collect.save(to_save, **kwargs)

    def find_one(self, spec_or_id=None, *args, **kwargs):
        """collection find_one method

        """
        wrapper = kwargs.pop('wrapper', False)
        if wrapper is True:
            return self._wrapper_find_one(spec_or_id, *args, **kwargs)

        return self.__collect.find_one(spec_or_id, *args, **kwargs)

    def find(self, *args, **kwargs):
        """collection find method

        """
        wrapper = kwargs.pop('wrapper', False)
        if wrapper is True:
            return self._wrapper_find(*args, **kwargs)

        return self.__collect.find(*args, **kwargs)

    @convert_to_record
    def _wrapper_find_one(self, spec_or_id=None, *args, **kwargs):
        """convert record to a dict that has no key error

        """
        return self.__collect.find_one(spec_or_id, *args, **kwargs)

    @convert_to_record
    def _wrapper_find(self, *args, **kwargs):
        """convert record to a dict that has no key error

        """
        return self.__collect.find(*args, **kwargs)

    def update(self, spec, document, multi=False, **kwargs):
        """collection update method

        """
        for opk in document.keys():
            if not opk.startswith('$') or opk not in self._operators:
                raise ValueError("invalid document update operator")

        if not document:
            raise ValueError("empty document update not allowed")           

        return self.__collect.update(spec, document, multi=multi, **kwargs)

    def remove(self, spec_or_id=None, **kwargs):
        """collection remove method
        warning:
            if you want to remove all documents,
            you must override _remove_all method to make sure
            you understand the result what you do
        """
        if isinstance(spec_or_id, dict) and spec_or_id == {}:
            raise ValueError("not allowed remove all documents")

        if spec_or_id is None:
            raise ValueError("not allowed remove all documents")

        return self.__collect.remove(spec_or_id, **kwargs)

    # gridfs method
    def put(self, value, **kwargs):
        """gridfs put method
        """
        if value:
            return self.__gridfs.put(value, **kwargs)

        return None

    def delete(self, _id):
        """gridfs delete method
        """
        return self.__gridfs.delete(self.to_objectid(_id))

    def get(self, _id):
        """gridfs get method
        """
        return self.__gridfs.get(self.to_objectid(_id))

    def read(self, _id):
        """gridfs read method
        """
        return self.__gridfs.get(self.to_objectid(_id)).read()

    def find_by_id(self, _id, column=None):
        """find record by _id
        """
        if isinstance(_id, list) or isinstance(_id, tuple):
            return (self.find_by_id(i, column) for i in _id if i)

        document_id = self.to_objectid(_id)

        if document_id is None:
            return None

        return self.__collect.find_one({'_id': document_id}, column)

    def remove_by_id(self, _id):
        if isinstance(_id, list) or isinstance(_id, tuple):
            return (self.remove_by_id(i) for i in _id)

        return self.__collect.remove({'_id': self.to_objectid(_id)})

    def find_and_modify(self, query=None, update=None, upsert=False, sort=None, full_response=False, **kwargs):
        return  self.__collect.find_and_modify(query=query, update=update, upsert=upsert, sort=sort, full_response=full_response, **kwargs)

    def ensure_index(self, key_or_list, cache_for=300, **kwargs):
        return self.__collect.ensure_index(key_or_list, cache_for=cache_for, **kwargs)

    def find_new_one(self, *args, **kwargs):
        cur = list(self.__collect.find(*args, **kwargs).limit(1).sort('_id', DESCENDING))
        if cur:
            return cur[0]

        return None

    def get_as_column(self, condition=None, column=None, skip=0, limit=0, sort=None):
        if column is None:
            column = self.column

        return self.find(condition, column, skip=skip, limit=limit, sort=sort)

    def get_as_dict(self, condition=None, column=None, skip=0, limit=0, sort=None):
        if column is None:
            column = self.column

        l = self.__collect.find(condition, column, skip=skip, limit=limit, sort=sort)

        as_dict, as_list = {}, []
        for i in l:
            as_dict[unicode(i['_id'])] = i
            as_list.append(i)

        return as_dict, as_list

    def create(self, record=None, **args):
        """init the new record
        function:
            创建一个新的record，并为数据一致性做初始化
        """
        if isinstance(record, list) or isinstance(record, tuple):
            for i in record:
                i = self._valid_record(i)

        if isinstance(record, dict):
            record = self._valid_record(record)

        return self.__collect.insert(record, **args)

    def _valid_record(self, record):
        if not isinstance(record, dict):
            raise Exception("%s record is not dict" % record)

        rset = set(record.keys())
        fset = set(self.field.keys())

        try:
            rset.remove('_id')
        except Exception as e:
            pass

        try:
            fset.remove('_id')
        except Exception as e:
            pass

        if not (fset ^ rset) <= fset:
            raise Exception("record keys is not equal to fields keys %s" % (list((fset ^ rset)-fset)))

        for k, v in self.field.items():
            if k not in record:
                if v[0] is datetime and not v[1]:
                    record[k] = self.datetime()
                    continue

                if v[0] is time and not v[1]:
                    record[k] = self.timestamp()
                    continue

                record[k] = v[1]

        return record

    def inc(self, spec_or_id, key, num=1):
        self.__collect.update(spec_or_id, {'$inc': {key: num}})


class BaseModel(BaseBaseModel):
    """全局公共业务方法

    """

    @classmethod
    def create_model(cls, name, field=None):
        """
        dynamic create new model
        :args field table field, if field is None or {}, this model can not use create method
        """
        if field:
            attrs = {'name': name, 'field': field}
        else:
            def create(self, *args, **kwargs):
                raise NotImplementedError()
            attrs = {'name': name, 'field': {'_id': ObjectId()}, 'create': create}

        return type(str(name), (cls, ), attrs)()



# -*- coding:utf-8 -*-

from datetime import datetime
import time

from bson.objectid import ObjectId
from pymongo import ASCENDING, DESCENDING

import turbo.model

from settings import MONGO_DB_MAPPING as _MONGO_DB_MAPPING

_PACKAGE_SPACE = globals()

class MixinModel(turbo.model.MixinModel):

    package_space = _PACKAGE_SPACE


class BaseModel(turbo.model.BaseModel, MixinModel):

    def __init__(self, db_name='test'):
        super(BaseModel, self).__init__(db_name, _MONGO_DB_MAPPING)


#-*- coding:utf-8 -*-

from tornado.options import define, options
import tornado.options

import setting
import turbo.register
import turbo.app

turbo.register.register_app(setting.SERVER_NAME, setting.TURBO_APP_SETTING, setting.WEB_APPLICATION_SETTING, __file__, globals())

define("port", default=8888, type=int)

if __name__ == '__main__':
    tornado.options.parse_command_line()
    turbo.app.start(options.port)


#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

from datetime import datetime
import copy
import time
import logging


from bson.objectid import ObjectId

from turbo.util import escape as es, camel_to_underscore
from turbo.test.util import unittest

class EscapeTest(unittest.TestCase):

    def setUp(self):
        child = {
                'id': ObjectId(), 
                'atime':datetime.now(), 
                'number': 10 , 
                'name':'hello world', 
                'mail':None,
            }
        self.record = {
                'id': ObjectId(), 
                'atime':datetime.now(), 
                'number': 10 , 
                'name':'hello world',
                'child': child, 
                'childs': [copy.deepcopy(child) for i in xrange(3)], 
            }

        self.values = [copy.deepcopy(self.record) for i in xrange(3)]

    def tearDown(self):
        del self.record
        del self.values

    def test_to_dict_str(self):
        self.check_value_type(es.to_dict_str(self.record))
    
    def test_default_encode(self):
        now = datetime.now()
        objid = ObjectId()
        number = 10
        self.assertEqual(es.default_encode(now), time.mktime(now.timetuple()))
        self.assertEqual(es.default_encode(objid), unicode(objid))
        self.assertEqual(es.default_encode(number), number)
    
    def test_recursive_to_str(self):
        now = datetime.now()
        objid = ObjectId()
        number = 10
        self.check_value_type(es.to_str(self.record))
        self.check_value_type(es.to_str(self.values))
        self.check_value_type(es.to_str(now))
        self.check_value_type(es.to_str(objid))
        self.check_value_type(es.to_str(number))
    
    def test_tobjectid(self):
        es.to_int('s')

    def test_json_encode(self):
        self.assertTrue(es.json_encode(es.to_str(self.values), indent=4) is not None)
    
    def test_json_decode(self):
        self.assertTrue(type(es.json_decode(
            es.json_encode(es.to_str(self.values), indent=4)
            )).__name__ == 'list'
        )
        
        #error test
        self.assertTrue(type(es.json_decode(
            es.json_encode(es.to_str(self.values), invlaid=4)
            )).__name__ == 'NoneType'
        )   


    def test_to_list_str(self):
        [self.check_value_type(v)  for v in es.to_list_str(self.values)]
    
    def test_camel_to_underscore(self):
        self.assertEqual(camel_to_underscore('HelloWorld'), 'hello_world')

    def check_value_type(self, v):
        if isinstance(v, list):
            [self.check_value_type(v1) for v1 in v]
            return

        if isinstance(v, dict):
            [self.check_value_type(v1) for k1, v1 in v.items()]
            return 

        self.assertTrue(self.check_base_value_type(v))

    def log(self, msg):
        logging.info(msg) 

    def check_base_value_type(self, v):
        return isinstance(v, int) or isinstance(v, float)  or isinstance(v, basestring) or v is None

if __name__ == '__main__':
    unittest.main()



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

import sys

import logging
import urllib

from bson.objectid import ObjectId

import turbo.httputil  as hu
from turbo.test.util import unittest



class HttpUtilTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass
        
    def test_encode_http_params(self):
        keyword = '美女'
        paras = hu.encode_http_params(k=10, h=2, key='ass', keyword=keyword, empty='')
        self.assertEqual(sorted(paras.split('&')),['h=2', 'k=10', 'key=ass', 'keyword=%s'%urllib.quote(keyword)])

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, print_function, with_statement

import os, time, base64
from copy import deepcopy
try:
    import cPickle as pickle
except ImportError:
    import pickle
try:
    import hashlib
    sha1 = hashlib.sha1
except ImportError:
    import sha
    sha1 = sha.new

from tornado.util import ObjectDict

from turbo.conf import app_config
from turbo.log import session_log


class Session(object):

    __slots__ = ['store', 'handler', 'app', '_data', '_dirty', '_config',
        '_initializer', 'session_id', '_session_object', '_session_name']

    def __init__(self, app, handler, store, initializer, session_config=None, session_object=None):
        self.handler = handler
        self.app = app
        self.store = store or DiskStore(app_config.store_config.diskpath)

        self._config = deepcopy(app_config.session_config)
        if session_config:
            self._config.update(session_config)

        self._session_name = self._config.name

        self._session_object = (session_object or CookieObject)(app, handler, self.store, self._config)
        
        self._data = ObjectDict()
        self._initializer = initializer

        self.session_id = None

        self._dirty = False
        self._processor()

    def __getitem__(self, name):
        if name not in self._data:
            session_log.error('%s key not exist in %s session' % (name, self.session_id))

        return self._data.get(name, None)

    def __setitem__(self, name, value):
        self._data[name] = value
        self._dirty = True

    def __delitem__(self, name):
        del self._data[name]
        self._dirty = True

    def __contains__(self, name):
        return name in self._data

    def __len__(self):
        return len(self._data)

    def __getattr__(self, name):
        return getattr(self._data, name)
    
    def __setattr__(self, name, value):
        if name in self.__slots__:
            super(Session, self).__setattr__(name, value)
        else:
            self._dirty = True
            setattr(self._data, name, value)
        
    def __delattr__(self, name):
        delattr(self._data, name)
        self._dirty = True

    def __iter__(self):
        for key in self._data:
            yield key

    def __repr__(self):
        return str(self._data)

    def _processor(self):
        """Application processor to setup session for every request"""
        self.store.cleanup(self._config.timeout)
        self._load()

    def _load(self):
        """Load the session from the store, by the id from cookie"""

        self.session_id = self._session_object.get_session_id()

        # protection against session_id tampering
        if self.session_id and not self._valid_session_id(self.session_id):
            self.session_id = None

        if self.session_id:
            d = self.store[self.session_id]
            if isinstance(d, dict) and d:
                self.update(d)

        if not self.session_id:
            self.session_id = self._session_object.generate_session_id()

        if not self._data:
            if self._initializer and isinstance(self._initializer, dict):
                self.update(deepcopy(self._initializer))

        self._session_object.set_session_id(self.session_id)

    def save(self):
        if self._dirty:
            self.store[self.session_id] = self._data

    def _valid_session_id(self, session_id):
        return True

    def kill(self):
        """Kill the session, make it no longer available"""
        del self.store[self.session_id]

    def clear(self):
        del self.store[self.session_id]


class SessionObject(object):


    def __init__(self, app, handler, store, session_config):
        self.app = app
        self.handler = handler
        self.store = store

        self._config = deepcopy(app_config.session_config)
        if session_config:
            self._config.update(session_config)

        self._session_name = self._config.name

    def get_session_id(self):
        raise NotImplementedError

    def set_session_id(self, session_id):
        raise NotImplementedError

    def clear_session_id(self):
        raise NotImplementedError

    def generate_session_id(self):
        """Generate a random id for session"""
        secret_key = self._config.secret_key
        while True:
            rand = os.urandom(16)
            now = time.time()
            session_id = sha1("%s%s%s%s" %(rand, now, self.handler.request.remote_ip, secret_key))
            session_id = session_id.hexdigest()
            if session_id not in self.store:
                break

        return session_id


class CookieObject(SessionObject):

    def set_session_id(self, session_id):
        self._set_cookie(self._session_name, session_id)

    def get_session_id(self):
        return self._get_cookie(self._session_name)

    def clear_session_id(self):
        self.handler.clear_cookie(self._session_name)

    def _set_cookie(self, name, value):
        cookie_domain = self._config.cookie_domain
        cookie_path = self._config.cookie_path
        cookie_expires = self._config.cookie_expires
        if self._config.secure:
            return self.handler.set_secure_cookie(name, value,
                expires_days=cookie_expires/(3600*24), domain=cookie_domain, path=cookie_path)
        else:
            return self.handler.set_cookie(name, value, expires=cookie_expires, domain=cookie_domain, path=cookie_path)
    
    def _get_cookie(self, name):
        if self._config.secure:
            return self.handler.get_secure_cookie(name)
        else:
            return self.handler.get_cookie(name)


class HeaderObject(SessionObject):

    def get_session_id(self):
        return self.handler.request.headers.get(self._session_name)

    def set_session_id(self, sid):
        self.handler.set_header(self._session_name, sid)

    def clear_session_id(self):
        self.handler.clear_header(self._session_name)


class Store(object):

    """Base class for session stores"""

    def __contains__(self, key):
        raise NotImplementedError

    def __getitem__(self, key):
        raise NotImplementedError

    def __setitem__(self, key, value):
        raise NotImplementedError

    def cleanup(self, timeout):
        """removes all the expired sessions"""
        raise NotImplementedError

    def encode(self, session_data):
        """encodes session dict as a string"""
        pickled = pickle.dumps(session_data)
        return base64.encodestring(pickled)

    def decode(self, session_data):
        """decodes the data to get back the session dict """
        pickled = base64.decodestring(session_data)
        return pickle.loads(pickled)


class DiskStore(Store):
    """
    Store for saving a session on disk.

        >>> import tempfile
        >>> root = tempfile.mkdtemp()
        >>> s = DiskStore(root)
        >>> s['a'] = 'foo'
        >>> s['a']
        'foo'
        >>> time.sleep(0.01)
        >>> s.cleanup(0.01)
        >>> s['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
    """
    def __init__(self, root):
        # if the storage root doesn't exists, create it.
        if not os.path.exists(root):
            os.makedirs(
                    os.path.abspath(root)
                    )
        self.root = root

    def _get_path(self, key):
        if os.path.sep in key: 
            raise ValueError, "Bad key: %s" % repr(key)
        return os.path.join(self.root, key)
    
    def __contains__(self, key):
        path = self._get_path(key)
        return os.path.exists(path)

    def __getitem__(self, key):
        path = self._get_path(key)
        if os.path.exists(path): 
            pickled = open(path).read()
            return self.decode(pickled)
        else:
            return ObjectDict()

    def __setitem__(self, key, value):
        path = self._get_path(key)
        pickled = self.encode(value)    
        try:
            f = open(path, 'w')
            try:
                f.write(pickled)
            finally: 
                f.close()
        except IOError:
            pass

    def __delitem__(self, key):
        path = self._get_path(key)
        if os.path.exists(path):
            os.remove(path)
    
    def cleanup(self, timeout):
        now = time.time()
        for f in os.listdir(self.root):
            path = self._get_path(f)
            atime = os.stat(path).st_atime
            if now - atime > timeout :
                os.remove(path)


class RedisStore(Store):

    def __init__(self, **kwargs):
        self.timeout = kwargs.get('timeout') or app_config.session_config.timeout

    def __contains__(self, key):
        return self.get_connection(key).exists(key)

    def __setitem__(self, key, value):
        conn = self.get_connection(key)
        conn.hset(key, 'data', self.encode(value))
        conn.expire(key, self.timeout)

    def __getitem__(self, key):
        data = self.get_connection(key).hget(key, 'data')
        if data:
            return self.decode(data)
        else:
            return ObjectDict()

    def __delitem__(self, key):
        self.get_connection(key).delete(key)

    def cleanup(self, timeout):
        pass

    def get_connection(self, key):
        import redis
        return redis.Redis()


#-*- coding:utf-8 -*-


class ResponseMsg(Exception):

    def __init__(self, code='', msg=''):
        self.code = code
        self.msg = msg 
        super(ResponseMsg, self).__init__(msg)

    def __str__(self):
        return '%s %s'%(self.code, self.msg)


class ResponseError(ResponseMsg):

    def __init__(self, code='', msg=''):
        super(ResponseError, self).__init__(code, msg)



# -*- coding:utf-8 -*-

import inspect
import os
import sys
from datetime import datetime, date
import time
import logging 
import json
from collections import Iterable

from bson.objectid import ObjectId

from turbo.log import util_log


def to_list_str(value):
    """recursively format list
    """
    result = []
    for index, v in enumerate(value):
        if isinstance(v, dict):
            result.append(to_dict_str(v))
            continue

        if isinstance(v, list):
            result.append(to_list_str(v))
            continue

        result.append(default_encode(v))

    return result


def to_dict_str(value):
    """recursively format dict
    """
    for k, v in value.items():
        if isinstance(v, dict):
            value[k] = to_dict_str(v)
            continue

        if isinstance(v, list):
            value[k] = to_list_str(v)
            continue

        value[k] = default_encode(v)

    return value


def default_encode(v):
    """数据类型转换
    """
    if isinstance(v, ObjectId):
        return unicode(v)

    if isinstance(v, datetime):
        return format_time(v)
    
    if isinstance(v, date):
        return format_time(v)

    return v


def to_str(v):
    if isinstance(v, dict):
        return to_dict_str(v)

    if isinstance(v, Iterable):
        return to_list_str(v)

    return default_encode(v)


def format_time(dt):
    """datetime format
    """
    return time.mktime(dt.timetuple())


def to_objectid(objid):
    """字符对象转换成objectid
    """  
    if objid is None:
        return objid
        
    try:
        objid = ObjectId(objid)
    except:
        util_log.error("%s is invalid objectid" % objid)
        return None
    
    return objid


def json_encode(data, **kwargs):
    try:
        return json.dumps(data, **kwargs)
    except Exception as e:
        util_log.error("Uncaught exception in json_encode", exc_info=True)


def json_decode(data, **kwargs):
    try:
        return json.loads(data, **kwargs)
    except Exception as e:
        util_log.error("Uncaught exception in json_decode", exc_info=True)


def to_int(value, default=None):
    try:
        return int(value)
    except ValueError as e:
        util_log.error(e)


def to_float(value, default=None):
    try:
        return float(value)
    except ValueError as e:
        util_log.error(e)


def to_datetime(t, micro=False):
    if micro:
        return datetime.fromtimestamp(t/1000)
    else:
        return datetime.fromtimestamp(t)


def to_time(t, micro=False):
    if micro:
        return time.mktime(t.timetuple())*1000
    else:
        return time.mktime(t.timetuple())


class Escape(object):
    
    __slots__ = ['to_list_str', 'to_dict_str', 'default_encode', 'format_time', 'to_objectid', 
        'to_str', 'to_time', 'to_datetime', 'to_int', 'to_float', 'json_decode', 'json_encode', '__gl']

    def __init__(self, module):
        self.__gl = module

    def __getattr__(self, name):
        if name in self.__slots__:
            return self.__gl.get(name)

        return getattr(self, name)


escape = Escape(globals())


def get_base_dir(currfile, dir_level_num=3):
    """
    find certain path according to currfile
    """
    root_path = os.path.abspath(currfile)
    for i in range(0, dir_level_num):
        root_path = os.path.dirname(root_path)

    return root_path


def join_sys_path(currfile, dir_level_num=3):
    """
    find certain path then load into sys path
    """
    if os.path.isdir(currfile):
        root_path = currfile
    else:
        root_path = get_base_dir(currfile, dir_level_num)
    
    sys.path.append(root_path)


def import_object(name, package_space=None):
    if name.count('.') == 0:
        return __import__(name, package_space, None)

    parts = name.split('.')
    obj = __import__('.'.join(parts[:-1]), package_space, None, [str(parts[-1])], 0)
    try:
        return getattr(obj, parts[-1])
    except AttributeError:
        raise ImportError("No module named %s" % parts[-1])


def camel_to_underscore(name):
    """
    convert CamelCase style to under_score_case
    """
    as_list = []
    length = len(name)
    for index, i in enumerate(name):
        if index !=0 and index != length-1 and i.isupper():
            as_list.append('_%s'%i.lower())
        else:
            as_list.append(i.lower())

    return ''.join(as_list)


def remove_folder(path, foldername):
    if not foldername:
        return 

    if not os.path.isdir(path):
        return

    dir_content = os.listdir(path)
    if not dir_content:
        return 

    for item in dir_content:
        child_path = os.path.join(path, item)

        if not os.path.isdir(child_path):
            continue

        if item != foldername:
            remove_folder(child_path, foldername)
            continue

        #os.rmdir can't be allowed to deldte not empty
        for root, dirs, files in os.walk(child_path, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))

            for name in dirs:
                os.rmdir(os.path.join(root, name))
        try:
            os.rmdir(child_path)
        except Exception as e:
            raise e


def remove_file(path, filename):
    if not filename:
        return 

    if not os.path.isdir(path):
        return

    dir_content = os.listdir(path)
    if not dir_content:
        return 

    for item in dir_content:
        child_path = os.path.join(path, item)

        if os.path.isdir(child_path):
            remove_file(child_path, filename)
            continue

        if item != filename:
            continue

        try:
            os.remove(child_path)
        except Exception as e:
            raise e


def remove_extension(path, extension):
    if not extension:
        return 

    if not os.path.isdir(path):
        return

    dir_content = os.listdir(path)
    if not dir_content:
        return 

    for item in dir_content:
        child_path = os.path.join(path, item)

        if os.path.isdir(child_path):
            remove_extension(child_path, extension)
            continue

        name, ext = os.path.splitext(item)

        if ext != extension:
            continue

        try:
            os.remove(child_path)
        except Exception as e:
            raise e



# -*- coding:utf-8 -*-

import os

from pymongo import (
    MongoReplicaSetClient,
    MongoClient,
    read_preferences
)
import gridfs

import setting

mc = MongoClient(host='localhost')

# test
test = mc['test']
test_files = gridfs.GridFS(mc['test_files'])

# user 
user = mc['user']
user_files = gridfs.GridFS(mc['user_files'])



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement


from turbo.test.util import unittest

from turbo.util import import_object

TEST_MODULES = [
    'escape_test',
    'basemodel_test',
    'log_test',
    'httputil_test',
    'app_test',
    'session_test',
    'util_test',
]


def main():
    testSuite = unittest.TestSuite()
    for module in TEST_MODULES:
        suite = unittest.TestLoader().loadTestsFromName(module)
        testSuite.addTest(suite)

    return testSuite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(main())



#-*- coding:utf-8 -*-

from datetime import datetime, timedelta

from pymongo import DESCENDING, ASCENDING

from models.user import model as user_model

from helpers import settings


MODEL_SLOTS = ['User']


class User(user_model.User):
    
    def hello_user(self):
        self.instance('user.User').find_one()



from __future__ import absolute_import, division, print_function, with_statement

try:
    basestring
except Exception as e:
    basestring = str

import urllib


def is_empty(v):
    if isinstance(v, basestring):
        if not v:
            return True
    if v is None:
        return True

    return False


def utf8(v):
    return v.encode('utf-8') if isinstance(v, unicode) else str(v)


def encode_http_params(**kw):
    '''
    url paremeter encode
    '''
    try:
        _fo = lambda k, v: '{name}={value}'.format(name=k, value=urllib.quote(v),)
    except:
        _fo = lambda k, v: '%s=%s' % (k,urllib.quote(v))

    _en = utf8 

    return '&'.join([_fo(_en(k), _en(v)) for k, v in kw.items() if not is_empty(v)])



import os
import sys
from tornado.util import ObjectDict

# server name
SERVER_NAME = '{{server_name}}'

# server dir
SERVER_DIR = os.path.dirname(os.path.abspath(__file__))
# project dir
PROJECT_DIR = os.path.dirname(SERVER_DIR)
sys.path.append(PROJECT_DIR)

# tornado web application settings
# details in http://www.tornadoweb.org/en/stable/web.html#tornado.web.Application.settings
WEB_APPLICATION_SETTING = ObjectDict(
    static_path=os.path.join(SERVER_DIR, "static"),
    template_path=os.path.join(SERVER_DIR, "templates"),
    xsrf_cookies=True,
    cookie_secret="xxx-xxx-xxx",
)

# turbo app setting 
TURBO_APP_SETTING = ObjectDict(
    log=ObjectDict(
        log_path=os.path.join("/var/log/", SERVER_NAME+'.log'),
        log_size=500*1024*1024,
        log_count=3,
    )
)

# check if app start in debug
if os.path.exists(os.path.join(SERVER_DIR, '__test__')):
    WEB_APPLICATION_SETTING['debug'] = True
    TURBO_APP_SETTING.log.log_path = os.path.join("", SERVER_NAME+'.log')


# -*- coding:utf-8 -*-
from __future__ import absolute_import, print_function

import os

try:
    basestring
except Exception as e:
    basestring = str

import tornado.web
import tornado.httpserver
import tornado.escape
import tornado.ioloop
from tornado.options import define, options
from tornado.util import ObjectDict

from pymongo import ASCENDING, DESCENDING
from bson.objectid import ObjectId

from turbo.core.exceptions import ResponseMsg, ResponseError
from turbo.util import escape as _es
import turbo.httputil as _ht 
from turbo.log import app_log
from turbo.conf import app_config
from turbo.session import Session, DiskStore


class Mixin(tornado.web.RequestHandler):

    @staticmethod
    def to_objectid(objid):
        return _es.to_objectid(objid)

    @staticmethod
    def to_int(value):
        return _es.to_int(value)

    @staticmethod
    def to_float(value):
        return _es.to_float(value)

    @staticmethod
    def to_bool(value):
        return bool(value)

    @staticmethod
    def to_str(v):
        return _es.to_str(v)

    @staticmethod
    def utf8(v):
        return tornado.escape.utf8(v)

    @staticmethod
    def encode_http_params(**kw):
        """
        url parameter encode
        """
        return _ht.encode_http_params(**kw)

    @staticmethod
    def json_encode(data, **kwargs):
        return _es.json_encode(data, **kwargs)

    @staticmethod
    def json_decode(data, **kwargs):
        return _es.json_decode(data, **kwargs)

    def is_ajax(self):
        return self.request.headers.get('X-Requested-With', None) == 'XMLHttpRequest'


class BaseBaseHandler(Mixin):
    """
    config request parameter like this:

    attrs:
        _get_params: need and option arguments for request hander like
                        _get_params = {
                                'need':[
                                    ('skip', int),
                                    ('limit', int),
                                ],
                                'option':[
                                    ('jsoncallback', basestring, None),
                                ]
                            }
        _post_params: the same to _get_params for post method

        _required_params: required arguments for all sub handler that element is three tuple like  [('skip', int, 0), ('limit', int, 0)]
        _get_required_params: the same to _required_params for get method
        _post_required_params: the same to _required_params for post method
        _put_required_params: the same to _required_params for put method 
        _delete_required_params: the same to _required_params for delete method
        _head_required_params: the same to _required_params for head method 
        _patch_required_params: the same to _required_params for patch method 
        _options_required_params: the same to _required_params for options method 
    
    """

    # override in subclass
    _required_params = [('skip', int, 0), ('limit', int, 0)]
    # override in subclass to extract the most need arguments
    
    _types = [ObjectId, None, basestring, str, int, float, list, file, bool]
    _data = None
    _session = None

    session_initializer = None 
    session_config = None
    session_object = None
    session_store = None # store for session

    def initialize(self):
        # app template path if exist must end with slash like user/
        # request context
        self.template_path = ''

    @property
    def session(self):
        if not self._session:
            self._session = Session(self.application, 
                self,
                self.session_store, 
                self.session_initializer, 
                self.session_config,
                self.session_object
            )

        return self._session

    def render(self, template_name, **kwargs):
        super(BaseBaseHandler, self).render('%s%s' % (self.template_path, template_name), context=self.get_context(), **kwargs)

    def sort_by(self, sort):
        return {1: ASCENDING, -1: DESCENDING}.get(sort, ASCENDING)

    # request context
    def get_context(self):
        return ObjectDict(session=self.session)

    # write output json
    def wo_json(self, data):
        callback = self.get_argument('jsoncallback', None)
        if callback:
            return self.write('%s(%s)' % (callback, self.json_encode(data)))

        self.write(self.json_encode(data))       

    # read in json
    def ri_json(self, data):
        return self.json_decode(data)

    @property
    def parameter(self):
        '''
        according to request method config to filter all request paremter
        if value is invalid then set None
        '''
        method = self.request.method.lower()
        arguments = self.request.arguments
        files = self.request.files

        rpd = {}  # request parameter dict

        def filter_parameter(key, tp, default=None):
            if tp not in self._types:
                raise ValueError("%s parameter expected types %s" % (key, self._types))

            if tp != file:
                if key not in arguments:
                    rpd[key] = default
                    return

                if tp in [ObjectId, int, float, bool]:
                    rpd[key] = getattr(self, 'to_%s' % getattr(tp, '__name__').lower())(self.get_argument(key))
                    return

                if tp == basestring or tp == str:
                    rpd[key] = self.get_argument(key, strip=False)
                    return

                if tp == list:
                    rpd[key] = self.get_arguments(key)
                    return

            if tp == file:
                if key not in files:
                    rpd[key] = []
                    return

                rpd[key] = self.request.files[key]                
        
        required_params = getattr(self, '_required_params', None)
        if isinstance(required_params, list):
            for key, tp, default in required_params:
                filter_parameter(key, tp, default)

        #extract method required params
        method_required_params = getattr(self, '_%s_required_params' % method, None)
        if isinstance(method_required_params, list):
            for key, tp, default in method_required_params:
                filter_parameter(key, tp, default)
        
        params = getattr(self, '_%s_params' % method, None)
        if params is None:
            return rpd

        #need arguments
        try:
            for key, tp in params.get('need', []):
                if tp == list:
                    filter_parameter(key, tp, [])
                else:
                    filter_parameter(key, tp)
        except ValueError as e:
            app_log.error('%s request need arguments parse error: %s' % (method, e))
            raise ValueError(e)
        except Exception as e:
            app_log.error('%s request need arguments parse error: %s' % (method, e))
            raise e

        #option arguments
        for key, tp, default in params.get('option', []):
            filter_parameter(key, tp, default)

        return rpd

    def head(self, *args, **kwargs):
        self._method_call('HEAD', *args, **kwargs)

    def get(self, *args, **kwargs):
        self._method_call('GET', *args, **kwargs)

    def post(self, *args, **kwargs):
        self._method_call('POST', *args, **kwargs)

    def delete(self, *args, **kwargs):
        self._method_call('DELETE', *args, **kwargs)

    def patch(self, *args, **kwargs):
        self._method_call('PATCH', *args, **kwargs)

    def put(self, *args, **kwargs):
        self._method_call('PUT', *args, **kwargs)

    def options(self, *args, **kwargs):
        self._method_call('OPTIONS', *args, **kwargs)

    def _method_call(self, method, *args, **kwargs):
        api_call = getattr(self, method)
        try:
            api_call(*args, **kwargs)
        except ResponseMsg as e:
            resp = self.init_resp(e.code, e.msg)
        except tornado.web.HTTPError as e:
            raise e
        except Exception as e:
            app_log.error('Uncaught Exception in %s %s call'%(getattr(getattr(self, '__class__'), '__name__'), method), exc_info=True)
            resp = self.init_resp(1, 'Unknown Error')
        else:
            resp = self.init_resp()

        self.wo_resp(resp)

    @staticmethod
    def init_resp(code=0, msg=None):
        """
        responsibility for rest api code msg
        can override for other style 

        :args code 0, rest api code 
        :args msg None, rest api msg

        """
        resp = {
            'code': code,
            'msg': msg,
            'res': {},
        }
        return resp

    def wo_resp(self, resp):
        """
        can override for other style
        """
        if isinstance(self._data, dict):
            resp['res'].update(self._data)

        return self.wo_json(resp)

    def HEAD(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def GET(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def POST(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def DELETE(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def PATCH(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def PUT(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def OPTIONS(self, *args, **kwargs):
        raise tornado.web.HTTPError(405)

    def route(self, route, *args, **kwargs):
        getattr(self,  "do_%s"%route, lambda *args, **kwargs: None)(*args, **kwargs)

    def on_finish(self):
        self._processor()
        super(BaseBaseHandler, self).on_finish()

    def _processor(self):
        """
        can override in son class to do some clean work after request
        """
        session_save = getattr(self.session, 'save', None)
        if session_save:
            session_save()


class BaseHandler(BaseBaseHandler):
    pass


class ErrorHandler(tornado.web.RequestHandler):

    def initialize(self, status_code):
        self.set_status(status_code)

    def prepare(self):
        t = tornado.template.Template('<h1>{{error_code}}</h1>')
        self.write(t.generate(error_code=self._status_code))
        self.finish()


def start(port=8888):
    app_log.info(app_config.app_name+' app start')
    app_config.error_handler = app_config.error_handler if app_config.error_handler else ErrorHandler
    tornado.web.ErrorHandler = app_config.error_handler
    application = tornado.web.Application(app_config.urls, **app_config.web_application_setting)
    http_server = tornado.httpserver.HTTPServer(application, xheaders=True)
    http_server.listen(port)
    ioloop = tornado.ioloop.IOLoop.instance()
    ioloop.start()



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

import os
import sys
import random
import time
import threading
import logging

from turbo import log
from turbo.test.util import unittest

test_log = log.getLogger(__file__)

base_path = os.path.abspath(__file__)
base_name = os.path.basename(base_path)
test_log_name = '.'.join(base_path.split('/')[1:-1]+[base_name.split('.')[0]])


class GetLoggerTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_logger(self):
        # module file log
        self.assertEqual(test_log.name, test_log_name)

        # root logger
        self.assertEqual(log.getLogger(), logging.root)
        self.assertEqual(len(log.getLogger(log_path='root.log').handlers), 2)
        self.assertEqual(len(log.getLogger(log_path='root2.log').handlers), 2)
        self.assertEqual(len(log.getLogger(level=logging.WARNING, log_path='root3.log').handlers), 3)

        # normal logger no handlers
        logger_one = log.getLogger('logger.one')
        self.assertEqual(logger_one.name, 'logger.one')
        self.assertEqual(len(logger_one.handlers), 0)

        # file handler one level only have only handler
        logger_two = log.getLogger('logger.two', logging.DEBUG, 'logger_test.log')
        self.assertEqual(logger_two.level, logging.DEBUG)
        self.assertEqual(len(logger_two.handlers), 1)

        logger_two_2 = log.getLogger('logger.two', logging.DEBUG, 'logger_test2.log')
        self.assertEqual(logger_two_2.level, logging.DEBUG)
        self.assertEqual(len(logger_two.handlers), 1)
        
        logger_two_3 = log.getLogger('logger.two', logging.ERROR, 'logger_test3.log')
        self.assertEqual(logger_two_3.level, logging.ERROR)
        self.assertEqual(len(logger_two.handlers), 2)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, print_function, with_statement

import os
import logging

from turbo.conf import app_config
from turbo.util import join_sys_path, get_base_dir, import_object
import turbo.helper
from turbo.helper import install_helper


def _install_app(package_space):
    for app in getattr(import_object('apps.settings', package_space), 'INSTALLED_APPS'):
       _ = import_object('.'.join(['apps', app]), package_space)        


def register_app(app_name, app_setting, web_application_setting, mainfile, package_space):
    """insert current project root path into sys path
    """
    from turbo import log
    app_config.app_name = app_name
    app_config.app_setting = app_setting
    app_config.project_name = os.path.basename(get_base_dir(mainfile, 2))
    app_config.web_application_setting = web_application_setting
    if app_setting.get('session_config'):
        app_config.session_config.update(app_setting['session_config'])
    log.getLogger(**app_setting.log)
    _install_app(package_space)


def register_url(url, handler, name=None, kwargs=None):
    """insert url into tornado application handlers group
    
    :arg str url: url 
    :handler object handler: url mapping handler 
    :name reverse url name
    :kwargs dict tornado handler initlize args
    """
    if name is None and kwargs is None:
        app_config.urls.append((url, handler))
        return

    if name is None:
        app_config.urls.append((url, handler, kwargs))
        return

    app_config.urls.append((url, handler, kwargs, name))
    

def register_group_urls(prefix, urls):
    for item in urls:
        url, handler = item[0:2]
        register_url(prefix+url, handler, *item[2:])


# -*- coding:utf-8 -*-

import tornado.web
import turbo.app
from turbo import app_config
from turbo.core.exceptions import ResponseError, ResponseMsg


class MixinHandler(turbo.app.BaseHandler):
    pass


class BaseHandler(MixinHandler):
    
    def initialize(self):
        super(BaseHandler, self).initialize()
        self._params = self.parameter

    def prepare(self):
        super(BaseHandler, self).prepare()

    def response_msg(self, msg='', code=1):
        raise ResponseMsg(code, msg)

    def response_error(self, msg='', code=1):
        raise ResponseError(code, msg)

    def http_error(self, status_code=404):
        raise tornado.web.HTTPError(status_code)

    def write_error(self, status_code, **kwargs):
        """Override to implement custom error pages.
        http://tornado.readthedocs.org/en/stable/_modules/tornado/web.html#RequestHandler.write_error
        """
        super(BaseHandler, self).write_error(status_code, **kwargs)


class ErrorHandler(BaseHandler):

    def initialize(self, status_code):
        super(ErrorHandler, self).initialize()
        self.set_status(status_code)
 
    def prepare(self):
        if not self.is_ajax():
            if self.get_status() == 404:
                raise self.http_error(404)
        else:
            self.wo_resp({'code': 1, 'msg': 'Api Not found'})
            self.finish()
            return

    def check_xsrf_cookie(self):
        # POSTs to an ErrorHandler don't actually have side effects,
        # so we don't need to check the xsrf token.  This allows POSTs
        # to the wrong url to return a 404 instead of 403.
        pass


from turbo.conf import app_config
app_config.error_handler = ErrorHandler


# -*- coding:utf-8 -*-

from db.conn import (
    test as _test,    
    user as _user,   

    test_files as _test_files, 
    user_files as _user_files, 
)

MONGO_DB_MAPPING = {
    'db': {
        'test': _test,
        'user': _user,
    },
    'db_file': {
        'test': _test_files,
        'user': _user_files,
    }
}



from __future__ import absolute_import, division, print_function, with_statement

import socket
import os
import signal
import sys
import random
import time
import threading
import logging
import requests
import multiprocessing
import time
from bson.objectid import ObjectId

from turbo.test.util import unittest

from turbo import app
from turbo.conf import app_config
from turbo import register

app_config.app_name = 'app_test'
app_config.web_application_setting = {
    'xsrf_cookies': False,
    'cookie_secret': 'adasfd' 
}

#logger = logging.getLogger()


class HomeHandler(app.BaseHandler):

    def get(self):
        assert self.is_ajax() == False
        self.write('get')

    def post(self):
        self.write('post')

    def put(self):
        self.write('put')


class ApiHandler(app.BaseHandler):

    _get_required_params = [
        ('skip', int, 0),
        ('limit', int, 20),
    ]

    _get_params = {
        'need': [
            ('action', None),
        ],
        'option': [
            ('who', basestring, 'python'),
            ('bool', bool, False),
            ('int', int, 0),
            ('float', float, 0),
            ('objectid', ObjectId, None),
            ('list', list, []),
        ]
    }

    _post_required_params = [
        ('skip', int, 0),
        ('limit', int, 20),
    ]

    _post_params = {
        'need': [
            ('who', basestring),
        ],
    }

    def GET(self):
        self._params = self.parameter

        assert self._params['skip'] == 0
        assert self._params['limit'] == 20
        assert self._params['who'] == 'python'
        assert self._params['action'] is None
        assert self._params['list'] == []
        assert self.is_ajax() == True

        self._data = {
            'value': self._params['who']
        }

    def POST(self):
        self._params = self.parameter
        print(self._params)

        assert self._params['skip'] == 0
        assert self._params['limit'] == 10

        self._data = {
            'value': self._params['who']
        }


    def PUT(self):
        self._data = {
            'api': {
                'put': 'value'
            }
        }

    def DELETE(self):
        raise Exception('value error')

    def wo_json(self, data):
        self.write(self.json_encode(data, indent=4))

PID = None
URL = None

def run_server(port):
    register.register_url('/', HomeHandler)
    register.register_url('', HomeHandler)
    register.register_url('/api', ApiHandler)
    app.start(port)


def is_used(port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    if sock.connect_ex(('localhost', port)) == 0:
        return True

    return False


def setUpModule():
    port = 8888
    while True:
        if not is_used(port):
            break
        port += 1
            
    server = multiprocessing.Process(target=run_server, args=(port,))
    server.start()
    global PID, URL
    URL = 'http://localhost:%s'%port
    PID = server.pid


def tearDownModule():
    os.kill(PID, signal.SIGKILL)


class AppTest(unittest.TestCase):

    def setUp(self):
        global URL 
        self.home_url = URL 
        self.api_url = URL + '/api'

    def test_get(self):
        resp = requests.get(self.home_url)
        self.assertEqual(resp.status_code, 200)

    def test_post(self):
        resp = requests.post(self.home_url)
        self.assertEqual(resp.status_code, 200)

    def test_get_api(self):
        resp = requests.get(self.api_url, headers={'X-Requested-With': 'XMLHttpRequest'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()['res']['value'], 'python')

    def test_delete_api(self):
        resp = requests.delete(self.api_url, headers={'X-Requested-With': 'XMLHttpRequest'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()['msg'], 'Unknown Error')

    def test_post_api(self):
        resp = requests.post(self.api_url, headers={'X-Requested-With': 'XMLHttpRequest'}, data={'limit': 10, 'who': 'ruby'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()['res']['value'], 'ruby')

    def test_404(self):
        resp = requests.get(self.home_url+'/hello')
        self.assertTrue(resp.content.find('404') != -1)

    def test_context(self):
        """TODO"""
        pass


if __name__ == '__main__':
    unittest.main()



#-*- coding:utf-8 -*-

import os
import sys
import unittest
import random
import time
import threading

from turbo.cache import object_cache


DATA = 'abcdfg'

THREAD_COUNT = 0

def load_data(value=None):
    while True:
        new_value = random.choice(DATA)
        if new_value != value:
            break

    return new_value

def log(msg):
    print msg

class ObjectCacheTestThread(threading.Thread):

    def __init__(self, oc, key, value, expire=5):
        super(ObjectCacheTestThread, self).__init__()
        self.oc = oc
        self.key = key
        self.expire = expire

    def run(self):
        start = time.time()
        while time.time() - start < self.expire:
            self.oc.get(self.key)

def object_cache_target(key, value):
    return object_cache.ObjectCache.get(key) == value

def trace_object_cache_run(func, key, value):
    global THREAD_COUNT
    if func(key ,value):
        THREAD_COUNT += 1

class ObjectCacheTest(unittest.TestCase):

    def setUp(self):
        self.oc = object_cache.ObjectCache()

    def tearDown(self):
        del self.oc

    def test_create(self):
        log('test create')
        key = self.oc.create(load_data, expire=30)
        self.assertTrue(key is not None)
        log('the key is {}'.format(key))
        log('test create end')
    
    def test_given_name(self):
        log('test given name')
        key = self.oc.create(load_data, 'cache_key', expire=30)
        self.assertTrue(key == 'cache_key')
        log('the key is {}'.format(key))
        log('test create end')
    
    def test_invalid_name(self):
        log('test invalid given name')
        with self.assertRaises(TypeError):
            key = self.oc.create(load_data, {'s': 10}, expire=30)
            self.assertEqual(key, None)

    def test_get(self):
        key = self.oc.create(load_data, expire=30)
        self.assertTrue(self.oc.get(key) in DATA)

    def test_expire(self):
        '''
        create object cache, wait until the value is expired
        make sure the new value is different to old value
        '''
        log('test expire start')
        expire = 5
        start = time.time()
        key = self.oc.create(load_data, expire=expire)
        old_value = self.oc.get(key)
        while time.time() - start < expire :
            log('{} in life period, the value is {}'.format(key,self.oc.get(key)))
            self.assertEqual(self.oc.get(key), old_value)
            time.sleep(1)

        log('{} is expired now'.format(key))

        self.assertTrue(self.oc.get(key, old_value) in DATA)
        self.assertTrue(self.oc.get(key) != old_value)
        
        log('{} new value is {}'.format(key, self.oc.get(key)))
        log('test expire end')

    def test_multi_thread(self):
        log('test multi thread start')
        key = self.oc.create(load_data)
        value = self.oc.get(key)
        threads_number = 3
        thread_list = []
        for i in range(threads_number):
            i = threading.Thread(target=trace_object_cache_run, args=(object_cache_target, key, value))
            thread_list.append(i)
            i.start()

        for i in thread_list:
            i.join()

        self.assertEqual(threads_number, THREAD_COUNT)

        log('test multi thread end')


if __name__ == '__main__':
    unittest.main()



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

import sys


if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

import sys
import signal
import random
import time
import logging
import requests
import multiprocessing
import time
import os


from turbo import app
from turbo.conf import app_config
from turbo import register
from turbo.session import RedisStore

app_config.app_name = 'app_test'
app_config.web_application_setting = {
    'xsrf_cookies': False,
    'cookie_secret': 'asdf/asdfiw872*&^2/'
}

from turbo.test.util import unittest


class HomeHandler(app.BaseHandler):

    session_initializer = {
        'time': time.time(),
        'uid': None,
    }

    def get(self):
        assert self.session.uid is None
        assert self.session.session_id is not None
        self.write('get')

    def post(self):
        self.session.uid = '7787'
        self.write('post')

    def put(self):
        assert self.session.uid == '7787'
        self.write('put')


class RedisStoreHandler(app.BaseHandler):

    session_initializer = {
        'time': time.time(),
        'uid': None,
    }

    session_store = RedisStore(timeout=3600)

    def get(self):
        assert self.session.uid is None
        assert self.session.session_id is not None
        self.write('get')

    def post(self):
        self.session.uid = '7787'
        self.write('post')

    def put(self):
        assert self.session.uid == '7787'
        self.write('put')


def run_server():
    register.register_url('/', HomeHandler)
    register.register_url('/redis', RedisStoreHandler)
    app.start()


class SessionTest(unittest.TestCase):

    def setUp(self):
        server = multiprocessing.Process(target=run_server)
        server.start()
        self.home_url = 'http://localhost:8888'
        self.redis_url = 'http://localhost:8888/redis'
        self.pid = server.pid
        time.sleep(1)

    def tearDown(self):
        os.kill(self.pid, signal.SIGKILL)
        
    def test_session(self):
        resp = requests.get(self.home_url, headers={'refer':'http://127.0.0.1:8888'})
        self.assertEqual(resp.status_code, 200)
        cookies = resp.cookies
        resp = requests.post(self.home_url, cookies=cookies)
        self.assertEqual(resp.status_code, 200)

        resp = requests.put(self.home_url, cookies=cookies)
        self.assertEqual(resp.status_code, 200)

    def test_redis_store_session(self):
        resp = requests.get(self.redis_url, headers={'refer':'http://127.0.0.1:8888'})
        self.assertEqual(resp.status_code, 200)
        cookies = resp.cookies
        resp = requests.post(self.redis_url, cookies=cookies)
        self.assertEqual(resp.status_code, 200)

        resp = requests.put(self.redis_url, cookies=cookies)
        self.assertEqual(resp.status_code, 200)

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, print_function, with_statement

from copy import deepcopy

class ObjectDict(dict):
    """Makes a dictionary behave like an object, with attribute-style access.
    """
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

        def __setattr__(self, name, value):
            self[name] = value


class AppConfig(object):
    
    _cookie_session_config = ObjectDict(
        name='session_id',
        cookie_domain=None,
        cookie_path='/',
        cookie_expires=86400,  # cookie expired  24 hours in seconds
        secure = True,
        secret_key='fLjUfxqXtfNoIldA0A0J', # generate session id,
        timeout=86400, # session timeout 24 hours in seconds
    )

    _store_config = ObjectDict(
        diskpath='/tmp/session',
    )

    def __init__(self):
        self.app_name = ''
        self.urls = []
        self.error_handler = None
        self.app_setting = {}
        self.web_application_setting = {}
        self.project_name = None
        self.session_config = self._cookie_session_config
        self.store_config = self._store_config

    @property
    def log_level(self):
        import logging
        level = self.app_setting.get('log', {}).get('log_level')
        if level is None:
            return logging.INFO

        return level

app_config = AppConfig()



from setuptools import setup, find_packages
import sys
import os


version = __import__('turbo').version


install_requires = [

]

for k in ['pymongo', 'requests', 'redis', 'docopt']:
    try:
        __import__(k)
    except ImportError:
        install_requires.append(k)

kwargs = {}

readme_path = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(readme_path, 'README.rst')) as f:
    kwargs['long_description'] = f.read()

if sys.version_info < (2, 7):
    install_requires.append('unittest2')
    install_requires.append('tornado<=4.3.0')
else:
    install_requires.append('tornado')

setup(
    name="turbo",
    version=version,
    author="Wecatch.me",
    author_email="wecatch.me@gmail.com",
    url="http://github.com/wecatch/app-turbo",
    license="http://www.apache.org/licenses/LICENSE-2.0",
    description="turbo is a web framework for fast web development based in tornado, mongodb, redis",
    keywords='web framework tornado mongodb',
    packages=find_packages(),
    install_requires=install_requires,
    scripts=['turbo/bin/turbo-admin'],
    **kwargs
)



#-*- coding:utf-8 -*-

from apps import base

import setting

class BaseHandler(base.BaseHandler):

    def initialize(self):
        super(BaseHandler,self).initialize()
        self.template_path = setting.TEMPLATE_PATH



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement


import os
import sys
import datetime
import json
import StringIO
import inspect


from turbo.model import BaseModel
from turbo.test.util import unittest
from bson.objectid import ObjectId

from pymongo import MongoClient
import gridfs

mc = MongoClient()


class Tag(BaseModel):

    name = 'tag'
    field = {
        'list':     (list, []),
        'imgid':    (ObjectId, None),
        'uid':      (ObjectId, None),
        'name':     (basestring, None),
        'atime':    (datetime.datetime, None),
        'up':       (dict, {}),
    }

    def __init__(self):
        db = {
            'db': {'test': mc['test']},
            'db_file': {'test': gridfs.GridFS(mc['test_files'])}
        }
        super(Tag, self).__init__('test', db)


class BaseModelTest(unittest.TestCase):

    def setUp(self):
        self.m = Tag()

    def tearDown(self):
        del self.m 

    def test_insert(self):
        pass

    def test_save(self):
        pass

    def test_find_one(self):
        pass

    def test_find(self):
        pass

    def test_update(self):
        with self.assertRaises(ValueError):
            self.m.update({},{'hello': 0}) 
        
        self.m.update({},{'$set':{'hellow': 0}})

        with self.assertRaises(ValueError):
            self.m.update({},{})

    def test_remove(self):
        pass

    def test_find_one_wrapper(self):
        # test find_one wrapper=True
        self.assertTrue(self.m.find_one(wrapper=True)['rd'] is None)
        
        # test find_one default wrapper=False
        with self.assertRaises(KeyError):
            self.m.find_one(wrapper=False)['rd']
        
        # test find_one wrapper= True and return None
        self.assertTrue(self.m.find_one({'_id':ObjectId()}, wrapper=True) is None)

    def test_find_wrapper(self):
        # test find wrapper=True return generator 
        one = self.m.find_one()
        with self.assertRaises(KeyError):
            one['keyerror']
        
        one = self.m.find_one(wrapper=True)
        self.assertEqual(one['keyerror'], None)
        
        for one in self.m.find(limit=5):
            with self.assertRaises(KeyError):
                one['keyerror']

        for one in self.m.find(limit=5, wrapper=True):
            self.assertEqual(one['keyerror'], None)


    def test_put(self):
        value = 'hello word'
        s = StringIO.StringIO()
        s.write(value)

        # put
        file_id = self.m.put(s.getvalue())
        self.assertTrue(isinstance(file_id, ObjectId))
        
        # get
        one = self.m.get(file_id)
        self.assertTrue(getattr(one,'read',False),'test get fail')
        self.assertEqual(one.read(),value)

    def test_delete(self):
        pass

    def test_find_by_id(self):
        pass

    def test_get_as_column(self):
        pass

    def test_get_as_dict(self):
        pass

    def test_to_objectid(self):
        self.assertTrue(self.m.to_objectid(None) is None)
        self.assertEqual(self.m.to_objectid('52c8fb6f1d41c820f1124350'), 
            ObjectId('52c8fb6f1d41c820f1124350'),'to_objectid is fail')

    def test_create_model(self):
        self.assertEqual(self.m.create_model('tag').find_one() is not None, True)

        with self.assertRaises(NotImplementedError):
            self.m.create_model('tag').create()
    
    def test_pymongo_collection_method(self):
        self.assertEqual(self.m.full_name, 'test.tag')
    
    def test_sub_collection(self):
        self.assertEqual(self.m.sub_collection('test').full_name, 'test.tag.test')

    def test_create(self):
        record = {
            'list': [
                {'key': ObjectId(),'key2': 'test','key3': ObjectId()},
                10,
                12,
                13,
                ['name', 'name', 'name', ObjectId(), ObjectId()],
                datetime.datetime.now(),
            ],
            'imgid':ObjectId(),
            'up':{
                'key1': ObjectId(),
                'key2': ObjectId(),
                'key3': ObjectId(),
            }
        }
        
        result = self.m.create(record)
        self.assertTrue(isinstance(result, ObjectId))

    def test_inc(self):
        pass

    def test_to_str(self):
        one = self.m.to_str(self.m.find(limit=10))
        self.assertTrue(isinstance(json.dumps(one), basestring))

    def test_to_one_str(self):
        one = self.m.to_one_str(self.m.find_one())
        self.assertTrue(isinstance(json.dumps(one), basestring))

    def test_default_encode(self):
        self.assertTrue(isinstance(self.m.default_encode(ObjectId()), basestring))
        self.assertTrue(isinstance(self.m.default_encode(datetime.datetime.now()),float))
        self.assertEqual(self.m.default_encode('string'), 'string')
    
    
    def test_get_as_dict(self):
        pass

    def log(self, one):
        print(one)

if __name__ == '__main__':
    unittest.main()



#-*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement


import os
import sys
import datetime
import json
import StringIO
import inspect
from copy import deepcopy


from turbo.model import BaseModel
from turbo.test.util import unittest
from turbo.util import escape
from bson.objectid import ObjectId


class EscapeTest(unittest.TestCase):


    def test_inc(self):
        pass

    def test_to_str(self):
        data = {
            'v1': 10,
            'v2': datetime.datetime.now(),
            'v3': ObjectId(),
            'v4': 'value',
        }
        
        self.assertTrue(isinstance(json.dumps(escape.to_str([deepcopy(data) for i in range(10)])), basestring))
        self.assertTrue(isinstance(json.dumps(escape.to_str(deepcopy(data))), basestring))


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, print_function, with_statement


import os
import logging
import logging.handlers

from turbo.conf import app_config


_formatter = logging.Formatter('%(levelname)s:%(asctime)s %(name)s:%(lineno)d:%(funcName)s %(message)s')


def _init_file_logger(logger, level, log_path, log_size, log_count):
    """
    one logger only have one level RotatingFileHandler 
    """
    if level not in [logging.NOTSET, logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL]:
        level = logging.DEBUG

    for h in logger.handlers:
        if isinstance(h, logging.handlers.RotatingFileHandler):
            if h.level == level:
                return

    fh = logging.handlers.RotatingFileHandler(log_path, maxBytes=log_size, backupCount=log_count)
    fh.setLevel(level)
    fh.setFormatter(_formatter)
    logger.addHandler(fh)
    

def _init_stream_logger(logger, level=None):
    ch = logging.StreamHandler()
    ch.setLevel(level or logging.DEBUG)
    ch.setFormatter(_formatter)
    logger.addHandler(ch)


def _module_logger(path):
    file_name = os.path.basename(path)
    module_name = file_name[0:file_name.rfind('.py')]
    logger_name_list = [module_name]

    # find project root dir util find it or to root dir '/'
    while True:
        # root path check
        path = os.path.dirname(path)
        if path == '/':
            break

        # project root path
        dirname = os.path.basename(path)
        if dirname == app_config.project_name:
            break
        logger_name_list.append(dirname)

    logger_name_list.reverse()

    return logging.getLogger('.'.join(logger_name_list))


def getLogger(currfile=None, level=None, log_path=None, log_size=500*1024*1024, log_count=3):
    # init logger first
    logger = None

    # py module logger
    if currfile is not None:
        path = os.path.abspath(currfile)
        if os.path.isfile(path):
            logger = _module_logger(path)
        elif isinstance(currfile, basestring) and currfile.strip():
            #normal logger
            logger = logging.getLogger(currfile)

        logger.setLevel(level or app_config.log_level)

    if not logger:
        logger = logging.getLogger()

    # keep the root logger at least have one streamhandler
    if not logger.root.handlers:
        _init_stream_logger(logger.root, level)

    if log_path:
        _init_file_logger(logger, level, log_path, log_size, log_count)

    return logger


# Logger objects for internal turbo use
app_log = getLogger('turbo.app')
model_log = getLogger('turbo.model')
util_log = getLogger('turbo.util')
helper_log = getLogger('turbo.helper')
session_log = getLogger('turbo.session')


#-*- coding:utf-8 -*-

from base import *

class User(Model):

    """
    email: user account
    passwd: user account passwd
    atime: added time
    """
    name = 'user'
    
    field = {
        'email':            (basestring, '')  ,
        'passwd':           (basestring, '')  ,
        'atime':            (datetime, None)    ,
    }




from __future__ import absolute_import, division, print_function, with_statement


import inspect
import sys

from turbo.util import import_object, camel_to_underscore
from turbo.log import helper_log
from turbo import model


class _HelperObjectDict(dict):

    def __setitem__(self, name, value):
        return super(_HelperObjectDict, self).setdefault(name, value)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise ValueError(name)


def install_helper(installing_helper_list, package_space):
    for item in installing_helper_list:
        # db model package
        package = import_object('.'.join(['helpers', item]), package_space)
        package_space[item] = _HelperObjectDict()
        # all py files  included by package
        all_modules = getattr(package, '__all__', [])
        for m in all_modules:
            try:
                module =  import_object('.'.join(['helpers',item, m]), package_space)
            except Exception as e:
                helper_log.error('module helpers.%s.%s Import Error'%(item, m), exc_info=True)
                sys.exit(0)

            for model_name in getattr(module, 'MODEL_SLOTS', []):
                model = getattr(module, model_name, None)
                if model:
                    camel_name = model.__name__
                    underscore_name = camel_to_underscore(camel_name)

                    package_space[item][underscore_name] = model()
                    package_space[item][camel_name] = model
